#!/usr/bin/python3
#
# Copy documentation from HTML file into C source

import os
import sys
import glob

def StripTags (sLine):
    sNew = ''
    bInTag = False
    for ch in sLine:
        if ( ch == '<' ):
            bInTag = True
        elif ( ch == '>' ):
            bInTag = False
        elif ( not bInTag ):
            sNew += ch
    return sNew

def Load (sName, sDocDir, sNewDir):
    with open (sName + '.c', 'r') as fSrc:
        with open (os.path.join (sDocDir, sName + '.html'), 'r') as fDoc:
            with open (os.path.join (sNewDir, sName + '.c'), 'w') as fNew:
                bDoc = True
                for sSrc in fSrc:
                    if ( bDoc and ( '(' in sSrc )):
                        bDoc = False
                        fNew.write ('/**\n')
                        mode = 0
                        for sDoc in fDoc:
                            sDoc = sDoc.strip ()
                            if ( sDoc.endswith ('</h3>') ):
                                sSep = '    +'
                                mode = 1
                            elif ( mode == 1 ):
                                if (( sDoc.startswith ('<p>Include file:') )
                                    or ( sDoc.startswith ('</body>') )):
                                    fNew.write ('*/\n\n');
                                    break;
                                if ( sDoc.startswith ('<table>') ):
                                    mode = 2
                                    sRowPart = ''
                                    table = [[], []]
                                elif ( sDoc != '' ):
                                    fNew.write ('    ')
                                    fNew.write (StripTags (sDoc))
                                    if ( sDoc.endswith ('</p>') ):
                                        fNew.write ('\n\n')
                                    else:
                                        fNew.write ('\n')
                            elif ( mode == 2 ):
                                if ( sDoc.startswith ('</table>') ):
                                    for nWth in table[0]:
                                        sSep += ('-' * nWth) + '+'
                                    sSep += '\n'
                                    fNew.write (sSep)
                                    for iRow, lRow in enumerate (table[1]):
                                        fNew.write ('    |')
                                        for iCol, sPart in enumerate (lRow):
                                            fNew.write (sPart.ljust (table[0][iCol]))
                                            fNew.write ('|')
                                        fNew.write ('\n')
                                        if ( iRow == 0 ):
                                            fNew.write (sSep)
                                    fNew.write (sSep)
                                    mode = 1
                                elif (sDoc.endswith ('</tr>') ):
                                    sRowPart += sDoc
                                    lParts = sRowPart.split ('><')
                                    lRow = []
                                    for sPart in lParts[1:-1]:
                                        lRow.append (sPart[3:-4].replace ('<br/>', '. '))
                                    table[1].append (lRow)
                                    for iCol, sPart in enumerate (lRow):
                                        nWth = len (sPart)
                                        if iCol < len (table[0]):
                                            if ( nWth > table[0][iCol] ):
                                                table[0][iCol] = nWth
                                        else:
                                            table[0].append (nWth)
                                    sRowPart = ''
                                else:
                                    sRowPart += sDoc
                    fNew.write (sSrc);
    
def LoadAll (sFind, sDocDir, sNewDir):
    for sFile in glob.glob (sFind):
        sBase = os.path.splitext (sFile)[0]
        if ( os.path.exists (os.path.join (sDocDir, sBase + '.html')) ):
            print (sFile)
            Load (sBase, sDocDir, sNewDir)

if ( len (sys.argv) == 4 ):
    LoadAll (sys.argv[1], sys.argv[2], sys.argv[3])
else:
    print ('Create copies of C source file containg documentation from an HTML file.\n')
    print ('Usage: LoadHelp.py <source files> <HTML folder> <destination folder>')
