#!/usr/bin/python3
#
# Turn source comments into HTML files
#
import  os
import  sys
import  glob
import  html

def InsBreak (sLine):
    sLine = html.escape (sLine)
    iFst = 0
    while (True):
        iStop = sLine.find ('. ', iFst)
        if ( iStop < 0 ):
            break
        if ( sLine[iStop-1] == '.' ):
            iFst = iStop + 2
        else:
            sLine = sLine[0:iStop+1] + '<br/>' + sLine[iStop+2:]
    return sLine

def InsNBSpc (sLine):
    sLine = html.escape (sLine)
    iFst = sLine.find ('(')
    while (True):
        iSpc = sLine.find (' ', iFst)
        if ( iSpc < 0 ):
            break
        if ( sLine[iSpc-1] == ',' ):
            iFst = iSpc + 1
        else:
            sLine = sLine[0:iSpc] + '&nbsp;' + sLine[iSpc+1:]
            iFst = iSpc + 6
    return sLine

class Topic:
    def __init__ (self, sFile):
        self.sBase, self.sExt = os.path.splitext (os.path.basename (sFile))
        self.html = ''
        self.include = None
        with open (sFile) as fIn:
            mode = 0
            bInPar = False
            sCurPar = None
            nInTab = 0
            nBrkOpn = 0
            nBrkCls = 0
            sList = ''
            for sLine in fIn:
                sLine = sLine.strip ()
                if ( sLine == '/**' ):
                    mode = 1
                elif ( sLine.startswith ('/*') ):
                    mode = 3
                elif ( sLine == '*/' ):
                    if ( sCurPar is not None ):
                        self.html += '</td></tr>\n</table>\n'
                        sCurPar = None
                    elif ( bInPar ):
                        self.html = self.html[0:-1] + '</p>\n'
                        bInPar = False
                    mode = 0
                elif ( sLine.startswith ('#include') ):
                    self.include = sLine[8:].strip ()[1:-1]
                elif ( mode == 1 ):
                    if ( sLine == '' ):
                        nClose = len (sList)
                        if ( nClose > 0 ):
                            self.html = self.html[0:-1] + '</li>\n' + ( '</ul>\n' * nClose )
                            sList = ''
                        if ( sCurPar is not None ):
                            self.html += '</td></tr>\n</table>\n'
                            sCurPar = None
                        elif ( bInPar ):
                            self.html = self.html[0:-1] + '</p>\n'
                            bInPar = False
                    elif ( sLine.startswith ('+-') ):
                        nInTab += 1
                        if ( nInTab == 1 ):
                            self.html += '<table>\n'
                        elif ( nInTab == 3 ):
                            self.html += '</table>\n'
                            nInTab = 0
                    elif ( sLine[0] == '|' ):
                        lPart = sLine.split ('|')
                        for iCol, sPart in enumerate (lPart):
                            lPart[iCol] = InsBreak (sPart.strip ())
                        if ( nInTab == 1 ):
                            self.html += '<tr><th>' + '</th><th>'.join (lPart[1:-1]) + '</th></tr>\n'
                        else:
                            self.html += '<tr><td>' + '</td><td>'.join (lPart[1:-1]) + '</td></tr>\n'
                    elif ( sLine[0] in '*+=-' ):
                        iList = sList.find (sLine[0])
                        if ( iList < 0 ):
                            sList += sLine[0]
                            self.html += '<ul>\n'
                        else:
                            self.html = self.html[0:-1] + '</li>\n'
                            nClose = len(sList) - iList - 1
                            if ( nClose > 0 ):
                                self.html += '</ul>\n' * nClose
                                sList = sList[0: iList + 1]
                        self.html += '<li>' + html.escape (sLine[1:].strip ()) + '\n'
                    elif ( '=' in sLine ):
                        sPar, sDef = sLine.split ('=', 1)
                        sPar = sPar.strip ()
                        sDef = sDef.strip ()
                        if ( sCurPar is None ):
                            self.html += ('<table class="param">\n<tr><td>' + sPar + ' =</td><td>'
                                          + html.escape (sDef))
                            sCurPar = sPar
                        elif ( sCurPar == sPar ):
                            self.html += '<br/>\n' + html.escape (sDef)
                        else:
                            self.html += ('</td></tr>\n<tr><td>' + sPar + ' =</td><td>'
                                          + html.escape (sDef))
                            sCurPar = sPar
                    elif ( sLine.lower ().startswith ('return:') ):
                        sPar, sDef = sLine.split (':', 1)
                        sPar = 'Return:'
                        sDef = sDef.strip ()
                        if ( sCurPar is None ):
                            self.html += ('<table class="param">\n<tr><td>' + sPar + '</td><td>'
                                          + html.escape (sDef))
                            sCurPar = sPar
                        elif ( sCurPar == sPar ):
                            self.html += '<br/>\n' + html.escape (sDef)
                        else:
                            self.html += ('</td></tr>\n<tr><td>' + sPar + '</td><td>'
                                          + html.escape (sDef))
                            sCurPar = sPar
                    else:
                        if ( sCurPar is not None ):
                            self.html += '</td></tr>\n</table>\n'
                            sCurPar = None
                        if (( not bInPar ) and ( sList == '' )):
                            self.html += '<p>'
                            bInPar = True
                        self.html += html.escape (sLine) + '\n'
                elif ((self.sExt == '.c' ) and ( mode == 0 ) and ( nBrkOpn == 0 ) and ( '(' in sLine )):
                    mode = 2
                    for nCh, ch in enumerate (sLine):
                        if ( ch == '(' ):
                            nBrkOpn += 1
                        elif ( ch == ')' ):
                            nBrkCls += 1
                            if ( nBrkCls == nBrkOpn ):
                                self.rtn = sLine[0:nCh+1]
                                mode = 0
                    if ( mode == 2 ):
                        self.rtn = sLine
                elif ( mode == 2 ):
                    self.rtn += ' '
                    for nCh, ch in enumerate (sLine):
                        if ( ch == '(' ):
                            nBrkOpn += 1
                        elif ( ch == ')' ):
                            nBrkCls += 1
                            if ( nBrkCls == nBrkOpn ):
                                self.rtn += sLine[0:nCh+1]
                                mode = 0
                    if ( mode == 2 ):
                        self.rtn += sLine

    def Save (self, sDir):
        with open (os.path.join (sDir, self.sBase + '.html'), 'w') as fOut:
            fOut.write ('<html>\n<head>\n<title>MTXLib - ' + self.sBase
                        + '</title>\n<link rel="stylesheet" href="help.css">\n'
                        ' </head>\n<body>\n')
            fOut.write ('<h3>' + InsNBSpc (self.rtn) + ';</h3>\n')
            fOut.write (self.html)
            if ( self.include is not None ):
                fOut.write ('<p>Include file: ' + self.include + '</p>\n')
            fOut.write ('</body>\n</html>\n')

class Include(Topic):
    def __init__ (self, sInclude):
        Topic.__init__ (self, os.path.join ('include', sInclude))

    def Save (self, fOut):
        fOut.write (self.html)

def MakeHelp (sFind, sDir, sIndex):
    dInclude = {}
    for sFile in glob.glob (sFind):
        print (sFile)
        t = Topic (sFile)
        if ( t.html != '' ):
            t.Save (sDir)
            if ( t.include is not None ):
                if ( t.include not in dInclude ):
                    dInclude[t.include] = []
                dInclude[t.include].append (t)
    if ( sIndex is None ):
        return
    lInclude = list (dInclude.keys ())
    lInclude.sort ()
    lIdx = []
    with open (os.path.join (sDir, sIndex), 'r') as fIdx:
        for sLine in fIdx:
            lIdx.append (sLine)
            if ( 'The Routines' in sLine ):
                break
    with open (os.path.join (sDir, sIndex), 'w') as fIdx:
        for sLine in lIdx:
            fIdx.write (sLine)
        for sInclude in lInclude:
            fIdx.write ('    <h3>Include file: "' + sInclude + '"</h3>\n')
            Include (sInclude).Save (fIdx);
            for t in dInclude[sInclude]:
                fIdx.write ('    <p class="routine"><a  href="' + t.sBase + '.html">'
                            + InsNBSpc (t.rtn) + ';</a></p>\n')
        fIdx.write ('  </body>\n</html\n')

if ( len (sys.argv) == 3 ):
    MakeHelp (sys.argv[1], sys.argv[2], None)
elif ( len (sys.argv) == 4 ):
    MakeHelp (sys.argv[1], sys.argv[2], sys.argv[3])
else:
    print ('Constructs help files from comments in C source.\n')
    print ('Usage: MakeHelp.py "<source files>" <doc folder> [<Index file>]')
