// ctccb.c - Register a callback routine for the CTC

#include <mtxint.h>

#ifndef __SDCCCALL
#define __SDCCCALL  0
#endif

/**
    Register a callback routine for the CTC. Note that only one routine can
    be installed for each channel. Also, setting a callback on channel 0 will
    halt the usual MTX BASIC processing of the 8ms interrupt such as maintaining
    the clock and moving sprites.

    This routine assumes that the CTC interrupt vector has not been changed
    from the default value 0f 0xFFF0

    chan =  CTC channel number.
    cb =    Callback routine (NULL to remove callback).

*/

void ctccb (unsigned char chan, void cb(void)) __naked
    {
    chan; cb;
    __asm
#if __SDCCCALL != 1
        ld      hl, #2       ; Pointer to parameters
        add     hl, sp
        ld      a, (hl)     ; Channel number
        inc     hl
        ld      e, (hl)     ; Routine address
        inc     hl
        ld      d, (hl)
#endif
        and     #3          ; Only 2 bits of channel number
        add     a, a        ; Multiply by 2
        push    af          ; Save 2 x channel number
        add     a, a        ; A = 4 x channel number
        add     a, a        ; A = 8 x channel number
        ld      b, #0       ; Offset into entry points
        ld      c, a
        ld      hl, #00010$
        add     hl, bc
        ld      a, d        ; Test for NULL callback
        or      e
        di                  ; Disable interrupts
        push    hl          ; Save location in entries table
        inc     hl          ; Skip the call to save registers
        inc     hl
        inc     hl
        jr      z, 00001$
        ld      (hl), #0xCD ; Call opcode
        inc     hl
        ld      (hl), e     ; Callback address
        inc     hl
        ld      (hl), d
        pop     de          ; Address of entry routine
        pop     af          ; 2 x channel number
        or      #0xF0       ; Location in vector table
        ld      l, a
        ld      h, #0xFF
        ld      (hl), e     ; Set vector
        inc     hl
        ld      (hl), d
        ei                  ; Re-enable interrupts
        ret
00001$:
        pop     de          ; Discard saved address
        ld      de, #00030$ ; Address of interrupt exit routine
        ld      (hl), #0xC3 ; Jump instruction
        inc     hl
        ld      (hl), e     ; Jump to interrupt exit
        inc     hl
        ld      (hl), d
        pop     af          ; Clear saved channel number
        ei                  ; Re-enable interrupts
        ret
00010$:
        call    00040$      ; Channel 0 entry routine
00011$:
        jp      00030$
        jr      00030$
        call    00040$      ; Channel 1 entry routine
        jp      00030$
        jr      00030$
        call    00040$      ; Channel 2 entry routine
        jp      00030$
        jr      00030$
        call    00040$      ; Channel 3 entry routine
        jp      00030$
00030$:
        pop     af          ; Restore registers after interrupt
        pop     iy
        pop     ix
        pop     de
        pop     bc
        pop     hl
        ei
        reti
00040$:
        ex      (sp), hl    ; Save registers for interrupt
        push    bc
        push    de
        push    ix
        push    iy
        push    af
        jp      (hl)
    __endasm;
    }
