//  findfirst.c - Find a directory entry

#include <diskio.h>

#ifndef __SDCCCALL
#define __SDCCCALL  0
#endif

/**
   Find a directory entry matching a file name, perhaps with wild cards.

   fcb =    File Control Block to use for the search.
   data =   Pointer to 128 byte buffer to receive directory entry.
   fn =     File name to find, optionally containing wildcards.
   Return:  Pointer to directory entry in buffer, or NULL if not found.
*/

void *findfirst (FCB *fcb, void *data, const char *fn) __naked
    {
    fcb; data; fn;
    __asm
        bdos = 0xD706
#if __SDCCCALL != 1
        ld      hl, #2      // Pointer to parameters
        add     hl, sp
        ld      e, (hl)     // FCB pointer
        inc     hl
        ld      d, (hl)
        inc     hl
        push    de
        ld      e, (hl)     // DMA pointer
        inc     hl
        ld      d, (hl)
        inc     hl
#endif
        push    hl
        call    mxl_setdma  // Set DMA address (if not NULL)
#if __SDCCCALL == 1
        pop     de          // FCB address
        pop     hl          // Return address
        ex      (sp), hl    // HL = File name pointer, return address back on stack
        ex      de, hl      // Pointers to correct registers
#else
        pop     hl          // Parameter pointer
        ld      e, (hl)     // File name pointer
        inc     hl
        ld      d, (hl)
        pop     hl          // FCB pointer
#endif
        push    hl          // Save the FCB pointer
        ld      a, d        // Test for file name
        or      e
        call    nz, mxl_fname   // Set file name
        pop     de          // DE = FCB pointer
        ld      c, #0x11    // Find first
        call    bdos
mxl_findend::
        inc     a           // Test for failure
        jr      z,00001$
        dec     a
        rrca                // Multiply by 32
        rrca
        rrca
        ld      c, a
        ld      b, #0
        call    _mxl_get_dma // Get DMA address
        add     hl, bc
#if __SDCCCALL == 1
        ex      de, hl      // Return value to DE
#endif
        ret
00001$:
#if __SDCCCALL == 1
        ld      de, #0      // Not found, return NULL
#else
        ld      hl, #0      // Not found, return NULL
#endif
        ret
    __endasm;
    }
