// fname.c - Copy a file name into an FCB with validation

#include <diskio.h>

#ifndef __SDCCCALL
#define __SDCCCALL  0
#endif

/**
   Initialise the file name in a File Control Block. High bit of any input characters
   is cleared, and lower case are converted to upper case. No other checks on valid
   characters is made. Individual character wildcard (?) are allowed, as is a global
   wildcard (*) at the end of the base name or the extention.

   This routine is called automatically by routines taking a file name as a parameter.

   fcb =    Pointer to file control block.
   fname =  Pointer to file name.
   return: 0 - OK. 1 - Invalid drive letter. 2 - Base name too long. 3 - Extension too long.
*/

unsigned char fname (FCB *fcb, const char *fn) __naked
    {
    fcb; fn;
    __asm
#if __SDCCCALL != 1
        ld      hl, #2      // Pointer to parameters
        add     hl, sp
        ld      e, (hl)     // FCB pointer
        inc     hl
        ld      d, (hl)
        inc     hl
        push    de
        ld      e, (hl)     // File name pointer
        inc     hl
        ld      d, (hl)
        pop     hl
#endif
mxl_fname::
        push    hl
        xor     a           // Initialise FCB
        ld      (hl), a
        inc     hl
        ld      a, #' '
        ld      b, #11
00001$:
        ld      (hl), a
        inc     hl
        djnz    00001$
        xor     a
        ld      b, #4
00002$:
        ld      (hl), a
        inc     hl
        djnz    00002$
        pop     hl
        inc     de          // Test for drive letter
        ld      a, (de)
        dec     de
        cp      #':'
        jr      nz, 00005$
        ld      a, (de)     // Drive letter
        inc     de
        inc     de
        call    00020$      // Convert to upper case
        cp      #'P' + 1     // Check valid
        jr      c, 00004$
00003$:
#if __SDCCCALL == 1
        ld      a, #1
#else
        ld      l, #1
#endif
        ret
00004$:
        sub     #'A'
        jr      c, 00003$
        inc     a
        ld      (hl), a
00005$:
        inc     hl
        ld      b, #8        // Length of file name
        call    00040$
        jr      nz, 00006$
        ld      b, #3       // Length of file extension
        call    00040$
        jr      nz, 00007$
        xor     a
#if __SDCCCALL != 1
        ld      l, a
#endif
        ret
00006$:
#if __SDCCCALL == 1
        ld      a, #2
#else
        ld      l, #2
#endif
        ret
00007$:
#if __SDCCCALL == 1
        ld      a, #3
#else
        ld      l, #3
#endif
        ret

00020$:
        and     #0x7F       // Clear high bit
        cp      #'a'         // Test for lower case
        ret     c
        cp      #'z' + 1
        ret     nc
        sub     #0x20       // Convert to upper case
        ret

00040$:
        ld      a, (de)     // File name or extension
        and     a
        jr      z, 00042$
        inc     de
        cp      #'.'
        jr      z, 00042$
        cp      #'*'
        jr      z, 00043$
        call    00020$      // Convert to upper case
        ld      (hl), a
        inc     hl
        djnz    00040$      // Repeat for each character
00041$:
        ld      a, (de)     // Get trailing character
        and     a
        ret     z
        inc     de
        cp      #'.'
        ret
00042$:
        inc     hl          // Advance pointer
        djnz    00042$
        ret
00043$:
        ld      a, #'?'     // Multiple wildcard
00044$:
        ld      (hl), a
        inc     hl
        djnz    00044$
        jr      00041$      // Check trailing character
    __endasm;
    }
