// diskio.h - CP/M disk I/O routines callable from SDCC

#ifndef H_DISKIO
#define H_DISKIO

/**
   BDOS level routines for performing file I/O and file operations.
   These routines read or write whole 128 byte logical disk sectors.

   Note: These routines call the SDX BDOS address, not the CP/M one.
*/

typedef struct s_fcb
    {
    unsigned char   dr;         //  Drive
    unsigned char   fn[8];      //  File name
    unsigned char   ft[3];      //  File type (extension)
    unsigned char   ex;         //  Current extent
    unsigned char   s1;
    unsigned char   s2;
    unsigned char   rc;         // Record count
    unsigned char   al[16];     // Allocation units
    unsigned char   cr;         // Current record
    unsigned int    rn;         // Record number
    unsigned char   r2;         // Record number overflow
    } FCB;

unsigned char fname (FCB *fcb, const char *fn);
unsigned char open (FCB *fcb, const char *fn);
unsigned char make (FCB *fcb, const char *fn);
unsigned char delete (FCB *fcb, const char *fn);
unsigned char close (FCB *fcb);
void setdma (void *addr);
unsigned char read (FCB *fcb, void *data);
unsigned char write (FCB *fcb, const void *data);
void *findfirst (FCB *fcb, void *data, const char *fn);
void *findnext (void);
unsigned char rename (FCB *fcb, const char *oldfn, const char *newfn);
unsigned char randrd (FCB *fcb, void *data, unsigned int recno);
unsigned char randwr (FCB *fcb, const void *data, unsigned int recno);
unsigned int numrec (FCB *, const char *fn);
unsigned int currec (FCB *);

#endif
