//  mtxint.h - Interrupt routines for the MTX

#ifndef H_MTXINT
#define H_MTXINT

/**
   The only source of interrupts in a standard MTX or SDX combination is the Counter Timer
   Chip (CTC). In MTX BASIC, channel 0 of the CTC is programmed to produce interrupts
   every 8ms. BASIC used these interrupts to:

   * Update the clock.
   * Display and move sprites.
   * Keyboard auto-repeat.
   * Testing for <Break> key pressed.
   * Updating sounds.
   * Optionally calling a user routine.

   Channels 1 and 3 are used when loading and saving audio tapes.

   Each of the four CTC channels may be used in either timer mode using the 4MHz system
   clock divided by either 16 or 256, or in counter mode. The counter inputs to each
   channel are:

   +---------+------------------------------+
   | Channel | Clock Source                 |
   +---------+------------------------------+
   |    0    | VDP Interrupt (end of frame) |
   |    1    | 4MHz / 13 clock              |
   |    2    | 4MHz / 13 clock              |
   |    3    | Tape input edge detect       |
   +---------+------------------------------+

   The Z80 is in interrupt mode 2, and the Interrupt vector table is located at 0xFFF0.
 */

void di (void);
void ei (void);
unsigned char getiff (void);
void int125 (void);
void intvdp (void);
void intrst (void);
void pause (unsigned int wait);
void ctccb (unsigned char chan, void cb(void));

#endif
