// mtxvdp.h - Routines for accessing the MTX Video Display Processor (VDP)

#ifndef H_MTXVDP
#define H_MTXVDP

/**
   Routines providing direct access to the VDP memory and registers.

   Because of the method of addressing VDP registers and memory, interrupts
   are disabled for at least part of the duration of each of these routines.
   They are re-enabled at routine exit if they were enabled on routine entry.

   MTX BASIC configures the VDP memory as follows:

   +---------+--------+-------------------------+
   | Address | Length | Description             |
   +---------+--------+-------------------------+
   | 0x0000  | 0x1800 | Graphics pattern table  |
   | 0x1800  | 0x0400 | Text pattern table      |
   | 0x1C00  | 0x03C0 | Text name table         |
   | 0x1FC0  | 0x0040 | Character colours table |
   | 0x2000  | 0x1800 | Graphics colours table  |
   | 0x3800  | 0x0400 | Sprite pattern table    |
   | 0x3C00  | 0x0300 | Graphics name table     |
   | 0x3F00  | 0x0080 | Sprite attributes table |
   +---------+--------+-------------------------+

*/

void vdpread (unsigned int addr, unsigned int len, void *data);
void vdpwrite (unsigned int addr, unsigned int len, void *data);
unsigned char vdpstatus (void);
void vdpreg (unsigned char reg, unsigned char val);
void vdpmode (unsigned char mode, unsigned char sprites, unsigned char ie);
unsigned char vdpaddr (unsigned char table, unsigned int addr);

#endif
