// intvdp.c - Configure VDP interrupt

#include <mtxint.h>

#ifndef __SDCCCALL
#define __SDCCCALL  0
#endif

/**
    Configure CTC channel 0 to produce an interrupt at the end of every VDP frame.
    In order to actually generate interrupts it is also necessary to set the IE
    (Interrupt Enable) bit in VDP register 1.

    After running this routine, pause(1) can be used to wait for the end of a VDP
    frame before updating the display.

    Note that after calling this routine, the clock will be updated and sprites
    moved every 20ms instead of every 8ms.

*/

void intvdp (void) __naked
    {
    __asm
        ctc0 = 0x08
        ld      a, #0xC5
        out     (ctc0), a       ; Counter mode, interrupt, time constant follows.
        ld      a, #1
        out     (ctc0), a       ; Interrupt on every VDP pulse
        ret
    __endasm;
    }
