//  numrec.c - Get the number of records in a file.

#include <diskio.h>

#ifndef __SDCCCALL
#define __SDCCCALL  0
#endif

/**
   Get the number of records in a file. The file should be closed (or at least not written).

   fcb =    Pointer to file control block.
   fn =     Pointer to file name. If NULL the FCB is assumed to contain a file name.
   return:  Number of records in a file.
*/

unsigned int numrec (FCB *fcb, const char *fn) __naked
    {
    fcb; fn;
    __asm
        bdos = 0xD706
#if __SDCCCALL != 1
        ld      hl, #2      // Pointer to parameters
        add     hl, sp
        ld      e, (hl)     // FCB pointer
        inc     hl
        ld      d, (hl)
        inc     hl
        push    de
        ld      e, (hl)     // File name pointer
        inc     hl
        ld      d, (hl)
        pop     hl
#endif
        push    hl          // Save the FCB pointer
        ld      a, d        // Test for file name
        or      e
        call    nz, mxl_fname   // Set file name
        pop     de          // Restore the FCB pointer
        and     a           // Test for error in file name
        jr      nz, 00001$
        push    de
        ld      c, #0x23    // Get file size
        call    bdos        // Perform the action
        pop     hl          // Restore the FCB pointer
        and     a           // Test for error
        jr      nz, 00001$
        ld      bc, #32     // Offset to record number
        add     hl, bc
        ld      e, (hl)     // Get record number
        inc     hl
        ld      d, (hl)
#if __SDCCCALL != 1
        ex      de, hl      // Result in HL
#endif
        ret
00001$:
#if __SDCCCALL == 1
        ld      de, #0      // Error, return zero
#else
        ld      hl, #0      // Error, return zero
#endif
        ret
    __endasm;
    }
