//  rename.c - Rename a file

#include <diskio.h>

#ifndef __SDCCCALL
#define __SDCCCALL  0
#endif

/**
   Rename a file.

   fcb =    File Control Block to use for the search.
   oldfn =  Original file name.
   newfn =  New file name.
   Return:  Zero on success, non-zero on failure
*/

unsigned char rename (FCB *fcb,  const char *oldfn,  const char *newfn) __naked
    {
    fcb; oldfn; newfn;
    __asm
        bdos = 0xD706
#if __SDCCCALL != 1
        ld      hl, #2      // Pointer to parameters
        add     hl, sp
        ld      e, (hl)     // FCB pointer
        inc     hl
        ld      d, (hl)
        inc     hl
        push    de
        ld      e, (hl)     // Old file name
        inc     hl
        ld      d, (hl)
        inc     hl
        ex      (sp), hl    // Restore FCB pointer and save parameter pointer
#endif
        push    hl          // Save FCB pointer
        ld      a, e        // Test for NULL filename
        or      d
        call    nz, mxl_fname   // Set old filename
        pop     hl
#if __SDCCCALL == 1
        ex      de, hl      // Save FCB pointer in DE
        pop     hl          // Return address
        ex      (sp), hl    // HL = Second file name. Return address back on stack.
        ex      de, hl      // Registers back in correct order
#else
        ex      (sp), hl    // Save FCB pointer. HL = Parameter pointer
        ld      e, (hl)     // New file name
        inc     hl
        ld      d, (hl)
        pop     hl          // Restore FCB pointer.
#endif
        and     a           // Test for error in first filename
        jr      nz, 00001$
        push    hl          // Save FCB pointer
        ld      bc, #16     // Second half of FCB
        add     hl, bc
        ld      a, e        // Test for NULL filename
        or      d
        call    nz, mxl_fname   // Set new filename
        pop     de          // DE = FCB pointer
        and     a           // Test for error in second filename
        jr      nz, 00002$
        ld      c, #0x17    // Rename file
        call    bdos
        and     #0xF0       // Test for error
00001$:
#if __SDCCCALL != 1
        ld      l, a
#endif
        ret
00002$:
        add     a, #8       // Error code for second file name
#if __SDCCCALL != 1
        ld      l, a
#endif
        ret
    __endasm;
    }
