//  sound3.7 - Set a sound ramp

#include <mtxsnd.h>

#ifndef __SDCCCALL
#define __SDCCCALL  0
#endif

/**
    Define a sound transition. The duration is specified in multiples of 16ms,
    and the frequency and volume increments are applied every 16ms.

    chan = Sound channel (0 - 3).
    freq = Frequency (0 - 8191).
    vol = Volume (0 - 1023).
    finc = Frequency increment.
    vinc = Volume increment.
    durn = Duration of transition.
    mode = 0 - Continue from previous frequency and volume (ignore new values).
    mode = 1 - Start at specified new frequency and volume.

*/

void sound7 (unsigned int chan, unsigned int freq, unsigned int vol,
			 unsigned int finc, unsigned int vinc, unsigned int durn,
			 unsigned int mode) __naked
    {
    chan; freq; vol; finc; vinc; durn; mode;
    __asm
        msss1 = 0x2D2F
        page = 0xFAD2
        temp = 0xFE04
#if __SDCCCALL == 1
        ld      (temp), hl      ; Save the tempnel number
        ld      (temp+2), de    ; Save the frequency
        ld      bc, #10         ; Data size
        ld      de, #temp+4     ; Sound system variables
#else
        ld      bc, #14         ; Data size
        ld      de, #temp       ; Sound system variables
#endif
        ld      hl, #2          ; Pointer to parameters
        add     hl, sp
        ldir                    ; Copy parameters to system variables
#if __SDCCCALL == 1
        pop     de              ; Get return address
        ld      sp, hl          ; Clean up stack
        push    de              ; Restore return address
#endif
        ld      a, (page)       ; Get current memory page
        push    af              ; Save it
        and     #0x0F           ; Select ROM 0
        ld      (page), a
        out     (0x00), a
        call    msss1           ; Process the sound
        pop     af              ; Restore original memory page
        ld      (page), a
        out     (0x00), a
        ret
    __endasm;
    }
