#!/usr/bin/python3
#
# Replacement linker script for SDCC

import os
import sys
import subprocess

class Linker:
    def __init__ (self):
        self.ldir = []
        self.libs = []
        self.objs = []
        self.out = None
        self.cpu = 'z80'
        self.cloc = '0x8000'
        self.dloc = None
        
    def Parse (self, args):
        mode = ''
        for arg in args:
            if ( mode == '' ):
                if ( arg.startswith ('-L') ):
                    if ( len (arg) > 2 ):
                        self.ldir.append (arg[2:])
                    else:
                        mode = 'L'
                elif ( arg.startswith ('-o') ):
                    if ( len (arg) > 2 ):
                        self.out = arg[2:]
                    else:
                        mode = 'o'
                elif ( arg.startswith ('-m') ):
                    if ( len (arg) > 2 ):
                        self.cpu = arg[2:]
                    else:
                        mode = 'm'
                elif ( arg == '--code-loc' ):
                    mode = 'cl'
                elif ( arg == '--data-loc' ):
                    mode = 'dl'
                elif ( not arg.startswith ('-') ):
                    if ( arg.endswith ('.lib') ):
                        self.libs.append (arg)
                    else:
                        self.objs.append (arg)
            elif ( mode == 'L' ):
                self.ldir.append (arg)
                mode = ''
            elif ( mode == 'o' ):
                self.out = arg
                mode = ''
            elif ( mode == 'm' ):
                self.cpu = arg
                mode = ''
            elif ( mode == 'cl' ):
                self.cloc = arg
                mode = ''
            elif ( mode == 'dl' ):
                self.dloc = arg
                mode = ''

    def Check (self):
        if ( self.out is None ):
            sys.stderr.write ('Error: No output file specified.')
            sys.exit (1)
        
    def Save (self):
        self.lkfn = os.path.splitext (self.out)[0] + '.lk'
        with open (self.lkfn, 'w') as fOut:
            fOut.write ('-mwxj\n')
            fOut.write ('-i {:s}\n'.format (self.out))
            fOut.write ('-b _CODE = {:s}\n'.format (self.cloc))
            if ( self.dloc is not None ):
                fOut.write ('-b _DATA = {:s}\n'.format (self.dloc))
            for dir in self.ldir:
                fOut.write ('-k {:s}\n'.format (dir))
                fOut.write ('-k {:s}/{:s}\n'.format (dir, self.cpu))
            for lib in self.libs:
                fOut.write ('-l {:s}\n'.format (lib))
            fOut.write ('-l {:s}\n'.format (self.cpu))
            for obj in self.objs:
                fOut.write ('{:s}\n'.format (obj))
            fOut.write ('\n-e\n')

    def Process (self):
        subprocess.run (['sdldz80', '-f', self.lkfn])

def Run ():
    ld = Linker ()
    ld.Parse (sys.argv[1:])
    ld.Check ()
    ld.Save ()
    ld.Process ()

Run ()
