// usercb.c - Sets routine called as part of MTX BASIC interrupt processing.

#include <mtxint.h>

#ifndef __SDCCCALL
#define __SDCCCALL  0
#endif

/**
    Sets routine called as part of MTX BASIC interrupt processing. By default this routine
    will be called every 8ms. If CTC channel 0 is re=programmed to pass through VDP
    interrupts, then the routine will be called every 20ms. If a direct callback on
    CTC channel 0 is defined, then BASIC interrupt processing, including this callback
    will not be processed.

    cb =    Routine to be called (NULL to remove callback).

*/

void usercb (void cb(void)) __naked
    {
    cb;
    __asm
       userint = 0xFA98
       intfff = 0xFD5E
#if __SDCCCALL == 1
        ex      de, hl      ; Callback pointer to DE.
#else
        ld      hl, #2      ; Parameter pointer
        add     hl, sp
        ld      e, (hl)
        inc     hl
        ld      d, (hl)
#endif
        ld      hl, #userint
        ld      a, d        ; Test for NULL pointer
        or      e
        di                  ; Disable interrupts
        jr      z, 00001$
        ld      (hl), #C3   ; Jump instruction
        inc     hl
        ld      (hl), e     ; Address of callback
        inc     hl
        ld      (hl), d
        ld      hl, #intfff ; Enable user interrupt
        set     4, (hl)
        ei                  ; Re-enable interrupts
        ret
00001$:
        ld      (hl), #0xC9 ; Just return on interrupt
        ld      hl, #intfff ; Disable user interrupt
        res     4, (hl)
        ei                  ; Re-enable interrupts
        ret
    __endasm;
    }
