// vdpmode.c - Configure VDP mode

#include <mtxvdp.h>

#ifndef __SDCCCALL
#define __SDCCCALL  0
#endif

/**
    Configure VDP mode. This is a convenience routine to set VDP registers 1 & 2.
    The address of the various VDP data structures (registers 2 - 6) will also
    need to be configured.

    mode =      Bits to form BLANK (Inverted), M3, M2 and M1
    mode =      0 - Graphics mode 1
    mode =      1 - Text mode
    mode =      2 - Multicolour mode
    mode =      4 - Graphics mode 2
    mode =      8 - Blank
    sprites =   0 - 8x8 sprites, magnification = 1
    sprites =   1 - 8x8 sprites, magnification = 2
    sprites =   2 - 16x16 sprites, magnification = 1
    sprites =   3 - 16x16 sprites, magnification = 2
    ie =        0 - Disable interrupt
    ie =        1 - Enable interrupt (if enabled in CTC)

*/

void vdpmode (unsigned char mode, unsigned char sprites, unsigned char ie) __naked
    {
    mode; sprites; ie;
    __asm
#if __SDCCCALL == 1
        ld      e, a        ; E = mode
        ld      d, l        ; D = sprites
        pop     hl          ; Return address
        pop     bc          ; C = ie
        dec     sp          ; Remove parameter from stack
        push    hl
#else
        pop     hl          ; Return address
        pop     de          ; E = mode, D = sprites
        pop     bc          ; C = ie
        push    bc          ; Restore the stack
        push    de
        push    hl
#endif
        ld      a, e        ; Set M3
        rrca
        and     #0x02
        ld      l, a
        ld      a, d        ; Restrict sprites to 0 - 3
        and     #0x03
        ld      d, a
        ld      a, c        ; Test for interrupt enable
        and     a
        jr      z, 00001$
        ld      a, #0x20
00001$:
        bit     0, e        ; Set M1
        jr      z, 00002$
        or      #0x10
00002$:
        bit     1, e        ; Set M2
        jr      z, 00003$
        or      #0x08
00003$:
        bit     3, e        ; Set BLANK
        jr      nz, 00004$
        or      #0x40
00004$:
        or      d           ; Sprite bits
        or      #0x80       ; 16K memory
        ld      e, a
        call    getiff      ; Test for interrupts enabled
        push    af
        di
        in      a,(0x02)    ; Read status register
        xor     a           ; Blank screen
        out     (0X02), a
        ld      a, #0x81    ; Register 1
        out     (0X02), a
        ld      a, l        ; Register 0
        out     (0x02), a
        ld      a, #0x80
        out     (0x02), a
        ld      a, e        ; Register 1
        out     (0x02), a
        ld      a, #0x81
        out     (0x02), a
        pop     af          ; Optionally enable interrupts
        ret     c
        ei
        ret
    __endasm;
    }
