// vdpwrite.c - Write data to VDP memory

#include <mtxvdp.h>

#ifndef __SDCCCALL
#define __SDCCCALL  0
#endif

/**
    Write data to VDP memory.

    addr =  VDP start address
    len =   Length of data to write
    data =  Address of buffer containing VDP data

*/

void vdpwrite (unsigned int addr, unsigned int len, void *data) __naked
    {
    addr; len; data;
    __asm
#if __SDCCCALL == 1
        ld      c, e        ; BC = Length
        ld      b, d
        ex      de, hl      ; DE = VDP address
        pop     hl          ; Return address
        ex      (sp), hl    ; HL = Source address
#else
        ld      hl, #2      ; Pointer to parameters
        add     hl, sp
        ld      e, (hl)     ; Get VDP address
        inc     hl
        ld      d, (hl)
        inc     hl
        push    de
        ld      c, (hl)     ; Get data length
        inc     hl
        ld      b, (hl)
        inc     hl
        ld      e, (hl)     ; Get source address
        inc     hl
        ld      d, (hl)
        ex      de, hl
        pop     de          ; Restore VDP address
#endif
        call    getiff      ; Test interrupt status
        push    af
        di
        in      a, (0x02)   ; Read status register
        ld      a, e        ; Output address
        out     (0x02), a
        ld      a, d
        and     #0x3F
        or      #0x40       ; Set write flag
        out     (0x02), a
00001$:                     ; Loop count = 57 T = 14us, Initial write 34 T = 8us
        ld      a, c        ;  4 T - Test for zero count
        or      b           ;  4 T
        jr      z, 00002$   ;  7 T
        dec     bc          ;  6 T - Count data
        ld      a, (hl)     ;  7 T - Get a byte
        inc     hl          ;  6 T
        out     (0x01), a   ; 11 T - Write it
        jr      00001$      ; 12 T
00002$:
        pop     af          ; Optionally restore interrupts
        ret     c
        ei
        ret
    __endasm;
    }
